/* print_kappa.c */
void print_kappa(void);

/* mask.c */
char *maskinfo(void);

/* print_hdr.c */
void print_header(void);

/* print_label.c */
void print_label(void);

/* print_mat.c */
void print_error_mat(int out_cols, int hdr);

/* print2csv_mat.c */
void print2csv_error_mat(int hdr);

/* print_json.c */
void print_json(void);

/* stats.c */
int stats(void);

/* calc_metrics.c */
void calc_metrics(void);
